import { getGlobal } from './utils';
import { ConcurrentRoot } from 'react-reconciler/constants';
let ReactInstance = null;
export function render(Component, $w, React, callback) {
    ReactInstance = React;
    self.reactVeloGlobals = {
        ReactInstance,
    };
    // the module is trying to access reactVeloGlobals as soon as it is initialized so needs to be delayed
    const reconciler = require('./reconciler').default;
    // @see https://github.com/facebook/react/blob/993ca533b42756811731f6b7791ae06a35ee6b4d/packages/react-reconciler/src/ReactRootTags.js
    // I think we are a legacy root?
    const rootContainer = { id: '~root~', type: 'root-container', $w, lastInstanceId: 0, instancesMap: new Map(), children: [] };
    const root = reconciler.createContainer(rootContainer, ConcurrentRoot, false, null, true);
    // Rule thinks we are in a React component, but we’re in a context that
    // only creates this value once instead.
    // eslint-disable-next-line react/jsx-no-constructed-context-values
    const renderContextValue = { reconciler };
    const RenderContext = ReactInstance.createContext(null);
    // callback is cast here because the typings do not mark that argument
    // as optional, even though it is.
    reconciler.updateContainer(ReactInstance.createElement(RenderContext.Provider, { value: renderContextValue }, ReactInstance.createElement(Component, null)), root, null, callback);
    getGlobal().performance && getGlobal().performance.mark('react-velo rendered');
    getGlobal().REACT_VELO_DEBUG && console.log('react-velo rendered');
}
const RepeaterWrapper = (props) => {
    const [ready, setReady] = ReactInstance.useState({});
    return ReactInstance.createElement('repeater', {
        id: props.id,
        onReadyItemId: (itemId, repeaterContext) => {
            setReady(prevReady => ({
                ...prevReady,
                [itemId]: repeaterContext
            }));
        },
        onRemovedItemId: (itemId) => {
            setReady(prevReady => ({
                ...prevReady,
                [itemId]: undefined,
            }));
        },
        data: props.data,
    }, props.data.map((item) => {
        if (typeof ready[item._id] !== 'undefined') {
            return ReactInstance.createElement('repeater-item', { key: item._id, repeaterContext: ready[item._id] }, props.renderItem(item));
        }
        return null;
    }));
};
function reactVeloComponentFactory(componentName, componentCache) {
    if (!componentCache.has(componentName))
        componentCache.set(componentName, ReactInstance.forwardRef(function SomeReactVeloComponet(props, ref) {
            // Guess it's a repeater if it has a renderItem prop
            if (props.renderItem) {
                return ReactInstance.createElement(RepeaterWrapper, { id: componentName, ...props });
            }
            return ReactInstance.createElement(componentName, { ...props, ref });
        }));
    return componentCache.get(componentName);
}
const reloHandler = {
    componentCache: new Map(),
    get(target, prop, receiver) {
        return reactVeloComponentFactory(prop, this.componentCache);
    }
};
// <W.MyAwesomeButton ... />
export const W = new Proxy({}, reloHandler);
// <V.button id="MyAwesomeButton" ... />
export const V = new Proxy({}, reloHandler);
//# sourceMappingURL=render.js.map